/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import Ubuntu.Components.Themes.Ambiance 0.1
import Ubuntu.Components.Popups 0.1
import "../common/ClockUtils.js" as Utils
import "../common"

// Page to edit or create a new alarm
/*
  Parameters passed:

  Variable to check if the page is for creating a new alarm or editing an existing one
  bool isNewAlarm

  Variable to hold the index of the alarm which is being edited. If page is called for creating a new alarm, this variable will not be available
  int alarmIndex
  */
Page {
    id: addAlarmPage
    objectName: "addAlarmPage"

    // Property to determine if this is New/Edit mode
    property bool isNewAlarm

    // Property to store the index of the alarm to be edited
    property int alarmIndex

    visible: false
    flickable: null

    Component.onCompleted: {
        if (isNewAlarm)
            addAlarmPage.title = i18n.tr("Add Alarm")
        else {
            addAlarmPage.title = i18n.tr("Edit Alarm")
            readAlarm()
        }
    }

    Alarm {
        id: alarm
        onStatusChanged: {
            if (status != Alarm.Ready)
                return;
            if ((operation > Alarm.NoOperation) && (operation < Alarm.Reseting)) {
                pagestack.pop();
            }
        }
    }

    // Function to save a new alarm
    function saveNewAlarm() {
        var alarmTime = new Date();

        alarmTime.setHours(Utils.hourstoCirclePosition(addAlarmFace.hours), Math.round(addAlarmFace.minutes), 0)

        alarm.message = nameAlarm.text
        alarm.date = alarmTime
        alarm.enabled = true
        alarm.save()
        if (alarm.error != Alarm.NoError) {
            Utils.log("Error saving alarm, code: " + alarm.error)
            addAlarmFace.innerLabel.visible = false
            if (alarm.error == 2) {
                errorLabel.text = i18n.tr("Please select a time in the future")
            } else {
                errorLabel.text = i18n.tr("Unable to set alarm")
            }
            errorLabel.visible = true
            errorTimer.start()
        }
    }

    // Function to read and show an existing alarm
    function readAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        alarm.message = temp_alarm.message
        alarm.type = temp_alarm.type
        alarm.daysOfWeek = temp_alarm.daysOfWeek
        alarm.enabled = temp_alarm.enabled
        alarm.date = temp_alarm.date

        addAlarmFace.hours = Utils.circlePositiontoHours(Qt.formatTime(alarm.date, "hh"))
        addAlarmFace.minutes = Qt.formatTime(alarm.date, "mm")
    }

    function updateAlarm() {
        var temp_alarm = alarmModel.get(alarmIndex)

        var alarmTime = new Date()

        alarmTime.setHours(Utils.hourstoCirclePosition(addAlarmFace.hours), Math.round(addAlarmFace.minutes), 0)

        temp_alarm.message = nameAlarm.text
        temp_alarm.type = alarm.type
        temp_alarm.daysOfWeek = alarm.daysOfWeek
        temp_alarm.date = alarmTime
        temp_alarm.save()
        pagestack.pop()
    }

    Timer {
        id: errorTimer
        interval: 1000
        running: true
        onTriggered: {
            errorLabel.visible = false
            addAlarmFace.innerLabel.visible = true
        }
    }

    Flickable {
        clip: true
        anchors.fill: parent
        contentHeight: mainColumn.height + units.gu(5)
        interactive: contentHeight > parent.height

        Column {
            id: mainColumn

            spacing: units.gu(1)
            anchors { left: parent.left; right: parent.right; top: parent.top; topMargin: units.gu(5)}

            ClockBase {
                id: addAlarmFace

                draggable: true
                showSecondHand: false

                onMinutesChanged: innerLabel.text = Utils.zeroleft(Utils.hourstoCirclePosition(hours), 2) + ":" + Utils.zeroleft(minutes, 2);
                onHoursChanged: innerLabel.text = Utils.zeroleft(Utils.hourstoCirclePosition(hours), 2) + ":" + Utils.zeroleft(minutes, 2);

                innerLabel.objectName: "labelAlarm"
                innerLabel.text: Utils.zeroleft(Utils.hourstoCirclePosition(hours), 2) + ":" + Utils.zeroleft(minutes, 2)

                anchors.horizontalCenter: parent.horizontalCenter

                Label {
                    id: errorLabel
                    visible: false
                    parent: addAlarmFace.centerItem
                    anchors {
                      centerIn: parent
                      fill: parent
                      margins: units.gu(2)
                    }
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    color: Theme.palette.normal.baseText

                    fontSize: "large"
                    text: i18n.tr("Unable to set alarm")
                    wrapMode: Text.WordWrap
                }
            }

            // Element to set the name of alarm
            LabelDots {
                id: nameAlarm
                objectName: "addAlarmName"

                text: alarm.message
                width: length < (addAlarmFace.width - units.gu(4))/units.gu(1.5)  ? length * units.gu(1.5) + units.gu(4) : addAlarmFace.width;
                placeholderText: i18n.tr("Alarm Name")
                anchors.horizontalCenter: addAlarmFace.horizontalCenter
                onClickedOutside: mainColumn.forceActiveFocus()
            }

            ListItem.ThinDivider { }

            Column {
                id: alarmOptions

                anchors { left: parent.left; leftMargin: units.gu(2); right: parent.right; rightMargin: units.gu(2) }
                spacing: units.gu(1)

                ListItem.Standard {
                    id: repeatSelector
                    anchors { left: parent.left; right: parent.right; margins: -units.gu(2) }
                    text: i18n.tr("Repeats")

                    control: Switch {
                        id: repeatSwitch
                        objectName: "RepeatSelector"
                        checked:  {
                            if (alarm.type == Alarm.OneTime)
                                return false;
                            else if (alarm.type == Alarm.Repeating) {
                                return true;
                            }
                        }
                        onCheckedChanged: {
                            switch (checked) {
                            case false:
                                alarm.type = Alarm.OneTime;
                                break;
                            case true:
                                alarm.type = Alarm.Repeating;
                                break;
                            }
                        }
                    }
                }

                OptionSelector {
                    id: occurSelector
                    objectName: "OccursSelector"
                    text: i18n.tr("Occurs")
                    visible: repeatSwitch.checked == false  // Show the day selector only when the alarm is a one-type alarm
                    containerHeight: itemHeight * 4
                    model: [Qt.locale().standaloneDayName(0), Qt.locale().standaloneDayName(1), Qt.locale().standaloneDayName(2), Qt.locale().standaloneDayName(3), Qt.locale().standaloneDayName(4), Qt.locale().standaloneDayName(5), Qt.locale().standaloneDayName(6)]

                    selectedIndex: alarm.date.getDay();

                    onSelectedIndexChanged: {
                        switch (selectedIndex) {
                        case 0:
                            alarm.daysOfWeek = Alarm.Sunday;
                            break;
                        case 1:
                            alarm.daysOfWeek = Alarm.Monday;
                            break;
                        case 2:
                            alarm.daysOfWeek = Alarm.Tuesday;
                            break;
                        case 3:
                            alarm.daysOfWeek = Alarm.Wednesday;
                            break;
                        case 4:
                            alarm.daysOfWeek = Alarm.Thursday;
                            break;
                        case 5:
                            alarm.daysOfWeek = Alarm.Friday;
                            break;
                        case 6:
                            alarm.daysOfWeek = Alarm.Saturday;
                            break;
                        }
                    }
                }

                ListItem.MultiValue {
                    id: multipleOccurSelector
                    anchors { left: parent.left; right: parent.right; margins: -units.gu(2) }
                    text: i18n.tr("Occurrence")
                    objectName: "OccurrenceList"
                    values: getValues()
                    visible: repeatSwitch.checked == true
                    onClicked: PopupUtils.open(Qt.resolvedUrl("AlarmDays.qml"), multipleOccurSelector, {"alarm": alarm});

                    function getValues() {
                        var occurs = [];
                        if (alarm.daysOfWeek & Alarm.Monday) occurs.push(Qt.locale().standaloneDayName(1));
                        if (alarm.daysOfWeek & Alarm.Tuesday) occurs.push(Qt.locale().standaloneDayName(2));
                        if (alarm.daysOfWeek & Alarm.Wednesday) occurs.push(Qt.locale().standaloneDayName(3));
                        if (alarm.daysOfWeek & Alarm.Thursday) occurs.push(Qt.locale().standaloneDayName(4));
                        if (alarm.daysOfWeek & Alarm.Friday) occurs.push(Qt.locale().standaloneDayName(5));
                        if (alarm.daysOfWeek & Alarm.Saturday) occurs.push(Qt.locale().standaloneDayName(6));
                        if (alarm.daysOfWeek & Alarm.Sunday) occurs.push(Qt.locale().standaloneDayName(7));
                        return occurs;
                    }
                }
            }
        }
    }

    // Toolbar to show back and save alarm button
    tools: ToolbarItems {
        id: toolbarAddAlarm

        opened: true
        locked: true

        ToolbarButton {
            id: addPresetToolbarButton
            objectName: "saveAlarmButton"
            iconSource: Qt.resolvedUrl("../images/select_icon.png")
            text: i18n.tr("Save")
            onTriggered: {
                if (addAlarmPage.isNewAlarm)
                    saveNewAlarm()
                else
                    updateAlarm()
            }
        }

    }
}

