/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 */

import QtQuick 2.0
import Ubuntu.Components 0.1
import Ubuntu.Components.ListItems 0.1 as ListItem
import QtQuick.XmlListModel 2.0
import "../common/ClockUtils.js" as Utils
import "../common"

Page {
    id: clockPage

    // Property to hold the formatted time string to show on the screen
    property string currentTimeFormatted
    property real currentUTCTime
    property real diff: new Date().getTimezoneOffset()
    property alias currentCity: currentLocationCity.text
    property alias currentLat: easterEggCircle.latitude
    property alias currentLng: easterEggCircle.longitude

    Component.onCompleted: {
        // TODO: Query GPS for location data
        Utils.log("ClockPage loaded");
        if (worldModel.city !== "undefined") {
            currentCity = worldModel.city;
            currentLng = worldModel.longitude;
            currentLat = worldModel.latitude;
        }
        Utils.log("Saved Current Location: " + worldModel.city);
        Utils.log("Saved Current Location Coordinates: " + currentLng + "," + currentLat)
        currentTimeFormatted = Utils.convertTime(new Date().getHours(), new Date().getMinutes(), new Date().getUTCSeconds(), appSetting.contents.timeFormat)
        updateTime();
    }

    actions: [
        Action {
            id: addCityAction
            text: i18n.tr("Add City")
            // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Add City) in the HUD
            keywords: i18n.tr("Add;Timezone;Timezones;World;City;Cities;Town;Towns;Place;Places;Location;Locations;Time;Locale;Local;Current")
            description: "Add a world city"
            iconSource: Qt.resolvedUrl("../images/add_icon.png")
            onTriggered: pagestack.push(Qt.resolvedUrl("WorldClock.qml"), {"isWorldCity": true})
        }
    ]

    // Function which runs every second to update the clock label
    function onTimerUpdate(now) {
        currentTimeFormatted = Utils.convertTime(now.getHours(), now.getMinutes(), now.getUTCSeconds(), appSetting.contents.timeFormat)
        clockFace.timeChanged(now)
        if (now.getUTCSeconds() == 0) {
            updateTime();
        }
    }

    // Function which runs every minute to update the world time
    function updateTime() {
        var now = new Date();
        now.setMinutes(now.getMinutes() + diff)
        currentUTCTime = now.getTime();
    }

    GeoIPModel {
        id: geoIP
        onStatusChanged: {
            if (status == XmlListModel.Ready && geoIP.get(0).city != worldModel.city && geoIP.get(0).city != "None") {
                Utils.log("Retrieved current location using GeoIP. Writing into database and setting it as current location")
                currentCity = geoIP.get(0).city
                currentLng = geoIP.get(0).lng
                currentLat = geoIP.get(0).lat
                worldModel.appendCurrentLocation(currentCity, currentLng, currentLat);
            }
            else if (status == XmlListModel.Ready && geoIP.get(0).city == "None") {
                if (worldModel.city == "undefined") {
                    currentCity = i18n.tr("Set current location name")
                    currentLocation.progression = true
                }
            }
            else if(status == XmlListModel.Error) {
                Utils.error("Unable to retrieve GeoIP Data -> " + geoIP.errorString())
                if (worldModel.city == "undefined") {
                    currentCity = i18n.tr("Set current location name")
                    currentLocation.progression = true
                }
            }
        }
    }

    Flickable {
        id: clockAnimationContainer
        objectName: "animationContainer"

        clip: true;
        anchors.fill: parent
        contentWidth: parent.width
        contentHeight: clockFace.height + clockFace.anchors.topMargin + savedWorldClock.height + savedWorldClock.anchors.topMargin + units.gu(3)

        // QML Element to draw the analogue clock face along with its hour, minute and second hands.
        ClockBase {
            id: clockFace

            anchors {
                top: parent.top
                topMargin: units.gu(10)
                horizontalCenter: parent.horizontalCenter
            }

            innerLabel.text: currentTimeFormatted
            innerLabel.font.pixelSize: appSetting.contents.timeFormat === "12-hour" ? units.dp(31): units.dp(41)

            hours: new Date().getHours() * 5
            minutes: new Date().getMinutes()
            seconds: new Date().getUTCSeconds()

            function timeChanged(now) {
                hours = now.getHours() * 5
                minutes = now.getMinutes()
                seconds = now.getUTCSeconds();
            }

            onClicked: {
                if (easterEggCircle.isReady == XmlListModel.Ready && worldModel.city !== "undefined")
                    clockFace.state == "" ? clockFace.state = "easteregg" : clockFace.state = "";
                else
                    Utils.error("Sunrise/Sunset times cannot be loaded without setting the current location or without a internet connection.")
            }

            EasterEgg {
                id: easterEggCircle
                parent: clockFace.centerItem
                anchors.centerIn: parent
                width: clockFace.centerItem.width; height: width;
                radius: width / 2;
            }

            states: [
                State {
                    name: "easteregg"
                    PropertyChanges { target: easterEggCircle; opacity: 1; }
                    PropertyChanges { target: clockFace.innerLabel; opacity: 0; }
                    PropertyChanges { target: easterEggCircle; sunriseLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunriseTime); }
                    PropertyChanges { target: easterEggCircle; sunsetLabel: easterEggCircle.getSunTime(easterEggCircle.model.get(0).sunsetTime); }
                },
                State {
                    name: ""
                    PropertyChanges { target: clockFace.innerLabel; opacity: 1; }
                    PropertyChanges { target: easterEggCircle; opacity: 0; }
                }
            ]
        }

        Column {
            id: savedWorldClock

            property int dynamicTopSpacing: clockPage.height - clockPage.header.height - clockFace.height - locationHeader.height - clockFace.anchors.topMargin - divider.height;

            height: childrenRect.height;
            anchors { left:parent.left; right:parent.right; top: clockFace.bottom; topMargin: savedWorldClock.dynamicTopSpacing }

            ListItem.ThinDivider { id: divider }

            ListItem.Header {
                id: locationHeader
                Label {
                    text: i18n.tr("Current Location")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListItem.Standard {
                id: currentLocation

                Label {
                    id: currentLocationCity
                    text: i18n.tr("Retrieving current location...")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }
                Label {
                    id: currentLocationTime
                    visible: worldModel.city !== "undefined" ? true : false
                    text: currentTimeFormatted
                    anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: currentLocation.progression == true ? units.gu(6) : units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "large"
                }

                onClicked: pagestack.push(Qt.resolvedUrl("WorldClock.qml"), {"isWorldCity": false})
            }

            ListItem.Header {
                Label {
                    text: i18n.tr("World")
                    anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(2) }
                    color: Theme.palette.normal.baseText
                    fontSize: "medium"
                }
            }

            ListView {
                id: listWorldClocks
                objectName: "listWorldClocks"
                
                clip: true
                anchors { left: parent.left; right: parent.right }
                height: 3 * units.gu(6) + units.gu(1)  // height is defined to show 3 items by default with a small margin
                model: worldModel
                currentIndex: -1

                delegate: ListItem.Standard {
                    objectName: "savedworldcity" + index
                    Label {
                        text: cityName
                        objectName: "city_name"
                        anchors { verticalCenter: parent.verticalCenter; left: parent.left; leftMargin: units.gu(3) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    Label {
                        text: appSetting.contents.timeFormat === "12-hour" ? Qt.formatTime(new Date(currentUTCTime + (rawOffSet * 60000)), "h:mm AP") : Qt.formatTime(new Date(currentUTCTime + (rawOffSet * 60000)), "hh:mm")
                        anchors { verticalCenter: parent.verticalCenter; right: parent.right; rightMargin: units.gu(2) }
                        color: Theme.palette.normal.baseText
                        fontSize: "large"
                    }

                    selected: listWorldClocks.currentIndex == index
                    removable: true
                    confirmRemoval: true

                    onItemRemoved: worldModel.removePreset(index);
                }
            }
        }
    }

    tools: ToolbarItems {
        id: toolbarClock

        ToolbarButton {
            id: settings
            action: appSettingsAction
        }

        ToolbarButton {
            id: addCity
            action: addCityAction
            objectName: "addCityAction"
        }
    }
}
