/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 */

import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import "../common/ClockUtils.js" as Utils

ListModel {
    id: model

    readonly property string dbName: "ubuntu-clock-app"
    readonly property string dbDescription: "World Clock"

    property string city: "undefined";
    property real longitude: 200;
    property real latitude: 200;

    property var _db: null

    function appendCurrentLocation(cityname, longitude, latitude)
    {
        try {
            _db.transaction(function(tx){
                var currentValue;
                var res = tx.executeSql('SELECT * FROM CurrentClock');

                if (res.rows.length > 0)
                    currentValue = res.rows.item(0).cityName;

                if (currentValue !== undefined)  // Update existing value or insert if none
                    res = tx.executeSql('UPDATE CurrentClock SET cityName = ?, longitude = ?, latitude = ?', [cityname, longitude, latitude]);
                else
                    res = tx.executeSql('INSERT INTO CurrentClock VALUES(?, ?, ?)', [cityname, longitude, latitude]);

                clockPage.currentCity = model.city = cityname
                clockPage.currentLng = model.longitude = longitude
                clockPage.currentLat = model.latitude = latitude
                Utils.log("Updating current location (manually)")
            });
        } catch (err) {
            Utils.error("Error setting current location '" + "': " + err);
            return false;
        }
    }

    function indexOf(cityName)
    {
        for(var i = 0; i < model.count; i++) {
            var element = model.get(i);

            if(element.cityName == cityName) {
                return i;            
            }
        }

        return -1;
    }

    function appendPreset(cityName, rawOffSet, longitude, latitude)
    {
        if (_appendDB(cityName, rawOffSet, longitude, latitude)) {
            var index = indexOf(cityName);
            var location = {"cityName" : cityName, "rawOffSet" : rawOffSet, "longitude" : longitude, "latitude" : latitude}

            if(index != -1) {
                model.set(index, location);       
            } else {
                model.append(location);                
            }
        }
    }

    function removePreset(index)
    {
        var cityName = get(index).cityName
        if (_removeDB(cityName)) {
            model.remove(index);
        }
    }

    function _createDB() {
        if (_db == null) return false;
        try {
            _db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS WorldClock(cityName TEXT, rawOffSet REAL, longitude REAL, latitude REAL)');
            });
        } catch (err) {
            Utils.error("Error creating WorldClock table in db:" + err);
            return false;
        }
        try {
            _db.transaction(function(tx){
                tx.executeSql('CREATE TABLE IF NOT EXISTS CurrentClock(cityName TEXT, longitude REAL, latitude REAL)');
            });
        } catch (err) {
            Utils.error("Error creating CurrentClock table in db:" + err);
            return false;
        }
        return true;
    }

    function _loadDB() {
        _db = LocalStorage.openDatabaseSync(model.dbName, "", model.dbDescription, 1000);
        if (_db == null) return false;
        if (!_createDB()) {
            _db = null;
            return false;
        }
        try {
            _db.readTransaction(function(tx){
                var res = tx.executeSql('SELECT * FROM WorldClock');
                if (res.rows.length > 0) {
                    for (var i = 0; i < res.rows.length; i++) {
                        model.append({"cityName" : res.rows.item(i).cityName,
                                         "rawOffSet": res.rows.item(i).rawOffSet,
                                         "longitude": res.rows.item(i).longitude,
                                         "latitude" : res.rows.item(i).latitude
                                     });
                    }
                }
            });
        } catch (err) {
            Utils.error("Error opening WorldClock database (" + err + ")");
            _db = null
            return false;
        }
        try {
            _db.readTransaction(function(tx){
                var currentstate = tx.executeSql('SELECT * FROM CurrentClock ORDER BY cityName');
                if (currentstate.rows.length > 0) {
                    city = currentstate.rows.item(0).cityName;
                    longitude = currentstate.rows.item(0).longitude;
                    latitude = currentstate.rows.item(0).latitude;
                }
            });
        } catch (err) {
            Utils.error("Error opening CurrentLocation database (" + err + ")");
            _db = null
            return false;
        }
        return true;
    }

    function _appendDB(cityName, rawOffSet, longitude, latitude) {
        if (_db == null) return false;

        try {
            _db.transaction(function(tx){
                var currentValue;
                var res = tx.executeSql('SELECT rawOffSet FROM WorldClock WHERE cityName=?', cityName);
                if (res.rows.length > 0) {
                    currentValue = res.rows.item(0).rawOffSet;
                }

                if (currentValue !== rawOffSet) {
                    // Update existing value or insert if none
                    if (currentValue !== undefined) {
                        res = tx.executeSql('UPDATE WorldClock SET rawOffSet = ?, longitude = ?, latitude = ? WHERE cityName = ?', [rawOffSet, longitude, latitude, cityName]);
                    } else {
                        res = tx.executeSql('INSERT INTO WorldClock VALUES(?, ?, ?, ?)', [cityName, rawOffSet, longitude, latitude]);
                    }
                }
            });
        } catch (err) {
            Utils.error("Error setting labeltxt '"+ label + "': " + err);
            return false;
        }
        return true;
    }

    function _removeDB(cityName) {
        if (_db == null) return false;

        try {
            _db.transaction(function(tx){
                tx.executeSql('DELETE FROM WorldClock WHERE cityName = ?', [cityName]);
            });
        } catch (err) {
            Utils.error("Error delete WorldClock'" + cityName + "': " + err);
            return false;
        }
        return true;
    }
}
