/*
 * Copyright (C) 2013 Canonical Ltd
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 3 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Authored by: Juha Ristolainen <juha.ristolainen@codemancers.fi>
 *              Nekhelesh Ramananthan <krnekhelesh@gmail.com>
 *              Nick Leppänen Larsson <frals@frals.se>
 *              Marco Biscaro <marcobiscaro2112@gmail.com>
 *              Alessandro Pozzi <signor.hyde@gmail.com>
 */

import QtQuick 2.0
import U1db 1.0 as U1db
import Ubuntu.Components 0.1
import "common/ClockUtils.js" as Utils
import "clock"
import "alarm"
import "stopwatch"
import "timer"

MainView {
    id: root

    // objectName for functional testing purposes (autopilot-qt5)
    objectName: "clock"

    ////////////////////////////////////////////////////////////////////////////////
    // Properties of this page
    applicationName: "com.ubuntu.clock"

    width: units.gu(50)
    height: units.gu(75)

    headerColor: "#57365E"
    backgroundColor: "#A55263"
    footerColor: "#D75669"

    // New properties of this page
    property var now

    // Initialization after page has loaded
    Component.onCompleted: {
        // Any init here
        Utils.log("MainView loaded");
    }

    actions: [
        Action {
            id: appSettingsAction
            text: i18n.tr("Settings")
            // TRANSLATORS: this is a list of semicolon-separated keywords to be used to activate this action (Settings) in the HUD
            keywords: i18n.tr("Settings;Setting;Configuration;")
            description: i18n.tr("Application Settings")
            iconSource: Qt.resolvedUrl("images/settings.svg")
            onTriggered: pagestack.push(Qt.resolvedUrl("common/SettingsPage.qml"))
        }
    ]

    focus: true
    Keys.onPressed: {
        if(event.modifiers && Qt.ControlModifier) {
            switch(event.key) {
            case Qt.Key_Tab:
                // Ctrl + Tab to switch to the next tab
                rootTabs.selectedTabIndex = rootTabs.selectedTabIndex === rootTabs.count-1 ? 0 : rootTabs.selectedTabIndex + 1
                event.accepted = true;
                break;

            case Qt.Key_Backtab:
                // Ctrl + Shift + Tab to switch to the previous tab
                rootTabs.selectedTabIndex = rootTabs.selectedTabIndex === 0 ? rootTabs.count-1 : rootTabs.selectedTabIndex - 1
                event.accepted = true
                break;

            case Qt.Key_N:
                // Ctrl + N to perform appropriate action depending on the current tab
                if(rootTabs.selectedTabIndex === 0)
                    pagestack.push(Qt.resolvedUrl("clock/WorldClock.qml"), {"isWorldCity": true})

                else if(rootTabs.selectedTabIndex === 1)
                    pagestack.push(Qt.resolvedUrl("alarm/AddAlarmPage.qml"), {"isNewAlarm": true})

                else if(rootTabs.selectedTabIndex === 2) {
                    timerPage.reset()
                    timerPage.state = "addPreset"
                }
                event.accepted = true
                break;
            }
        }
    }

    // Timer that runs our app
    Timer {
        id: updateTimer
        objectName: "updateTimer"

        interval: 1000
        repeat: true
        running: true

        onTriggered: {
            now = new Date();
            // FIXME: only send callback to pages that need it
            /* TODO: this timer should probably be changed to fire
              every minute (max) when the app is in the background,
              or just be activated when the app is visible in the
              switcher
              */
            clockPage.onTimerUpdate(now);
            timerPage.onTimerUpdate(now);

            /* stopwatchPage.onTimerUpdate(now) is not required since
               there is an inbuilt stopwatch timer which runs at a
               higher frequency.
             */
        }
    }

    // Database to store app data locally
    U1db.Database {
        id: db
        path: "settings"
    }

    // Document to store app settings data
    U1db.Document {
        id: appSetting
        database: db
        docId: "appSettings"
        create: true
        defaults: { "timeFormat": "12-hour", "style": "analogue" }
    }

    ////////////////////////////////////////////////////////////////////////////////
    // UI
    PageStack {
        id: pagestack

        Component.onCompleted: push(rootTabs);

        // Applications pages in tabs (Clock, Alarm, Timer, Stopwatch)
        Tabs {
            id: rootTabs
            objectName: "rootTabs"
            anchors.fill: parent

            Tab {
                objectName: "ClockTab"

                title: i18n.tr("Clock")

                // Tab content begins here
                page: ClockPage {
                    id: clockPage

                    WorldClockModel {
                        id: worldModel;
                        Component.onCompleted: {
                            _loadDB();
                            Utils.log("Clock Database loaded")
                        }
                    }
                }
            }

            // Disabled Alarm Tab until the feature is ready for use.
            Tab {
                objectName: "AlarmTab"

                title: i18n.tr("Alarm")

                page: AlarmPage {
                    id: alarmPage

                    AlarmModel {
                        id: alarmModel
                        Component.onCompleted: Utils.log("Alarm Database loaded")
                        onModelReset: alarmPage.get_next_active_alarm()
                    }
                }
            }

            Tab {
                objectName: "TimerTab"

                title: i18n.tr("Timer")

                page: TimerPage {
                    id: timerPage
                    PresetTimersModel {
                        id: presetModel
                        Component.onCompleted: Utils.log("Timer Preset Database loaded")
                    }
                }
            }

            Tab {
                objectName: "StopwatchTab"

                title: i18n.tr("Stopwatch")

                page: StopwatchPage {
                    id: stopwatchPage
                }
            }
        }
    }
}
